/*
 * Copyright (C) 2000-2005 Erik Andersen <andersen@uclibc.org>
 *
 * Licensed under the LGPL v2.1, see the file COPYING.LIB in this tarball.
 */

#define WANT_WIDE
#define __UCLIBC_DO_XLOCALE
#define L_strxfrm_l
#include "_collate.c"
