/* Set given exception flags.  x86_64 version.
   Copyright (C) 2016-2017 Free Software Foundation, Inc.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <fenv.h>

int
fesetexcept (int excepts)
{
  unsigned int mxcsr;

  __asm__ ("stmxcsr %0" : "=m" (*&mxcsr));
  mxcsr |= excepts & FE_ALL_EXCEPT;
  __asm__ ("ldmxcsr %0" : : "m" (*&mxcsr));

  return 0;
}
