/* Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Jakub Jelinek <jj@ultra.linux.cz>, 1999.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.

   Ported to uClibc by:
   Austin Foxley, Ceton Corporation <austinf@cetoncorp.com>
 */

#include <features.h>
#include <sys/syscall.h>
#include <asm/errno.h>

.text
.global	pipe
.type	pipe, @function
.align 4

pipe:
	save	%sp,-96,%sp

	/* sanity check arguments */
	tst	%i0
	be	.Lerror
	 mov	%i2,%o0

	/* Do the system call */
	set	__NR_pipe,%g1
	ta	0x10
	bcs	.Lerror
	 nop
 
    st %o0,[%i0]
    st %o1,[%i0+4]
	ret
	 restore %g0,%g0,%o0

.Lerror:
	call	__errno_location
	 or	%g0,EINVAL,%i0
	st	%i0,[%o0]
	ret
	 restore %g0,-1,%o0

.size pipe,.-pipe
libc_hidden_def(pipe)
