/* Copyright (C) 2015 Bernhard Reutner-Fischer
 * Licensed under the LGPL v2.1 or later, see the file COPYING.LIB in this tarball.
 */

const void *const __dso_handle attribute_hidden = &__dso_handle;
