/*
 * Copyright (C) 2013 Synopsys, Inc. (www.synopsys.com)
 *
 * Licensed under the LGPL v2.1 or later, see the file COPYING.LIB in this tarball.
 */

#ifndef _STACKINFO_H
#define _STACKINFO_H	1

/* On ARC, the stack grows down.  */
#define _STACK_GROWS_DOWN	1

#endif	/* stackinfo.h */
