/* Private macros for accessing __jmp_buf contents.  BFIN version.  */
/*
 * Copyright (C) 2000-2006 Erik Andersen <andersen@uclibc.org>
 *
 * Licensed under the LGPL v2.1, see the file COPYING.LIB in this tarball.
 */

#define JB_SP		1
