/* Private macros for accessing __jmp_buf contents.  avr32 version.
 * Copyright (C) 2004-2005 Atmel Corporation
 *
 * This file is subject to the terms and conditions of the GNU Lesser General
 * Public License.  See the file "COPYING.LIB" in the main directory of this
 * archive for more details.
 */

#define __JMP_BUF_SP	4
