/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2017 Kalray Inc.
 */

#ifndef _UAPI_ASM_KVX_POSIX_TYPES_H
#define _UAPI_ASM_KVX_POSIX_TYPES_H

#include <asm-generic/posix_types.h>

#endif /* _UAPI_ASM_KVX_POSIX_TYPES_H */
