/*
 * Copyright (C) 2016 Andes Technology, Inc.
 * Licensed under the LGPL v2.1, see the file COPYING.LIB in this tarball.
 */

#ifndef _SYS_ELF_H
#define _SYS_ELF_H	1

#warning "This header is obsolete; use <sys/procfs.h> instead."

#include <sys/procfs.h>

#endif	/* _SYS_ELF_H */
