/*
 * Copyright (C) 2017 Hangzhou C-SKY Microsystems co.,ltd.
 *
 * Licensed under the LGPL v2.1 or later, see the file COPYING.LIB
 * in this tarball.
 */

#ifndef _STACKINFO_H
#define _STACKINFO_H	1

#define _STACK_GROWS_DOWN	1

#endif
