/*
 * Copyright (C) 2018 Kalray Inc.	
 * Licensed under the LGPL v2.1, see the file COPYING.LIB in this tarball.
 */

#warning NOT IMPLEMENTED: THIS IS A SKELETON

 .text
 .globl _dl_linux_resolve
 .type _dl_linux_resolve, %function
 .align 2

_dl_linux_resolve:
	errop
	;;

.size _dl_linux_resolve, .-_dl_linux_resolve
